@@ -23,16 +23,25 @@ _initial_run_time_ns = None
 
 def analyze_runtime_failure_and_heal(stderr: str, cmd_args: list, config_manager: ConfigManager):
     """
-    Analyzes stderr for a broader range of version conflict errors and triggers healing.
-    This function now correctly returns a tuple (exit_code, stats) in all cases.
+    Analyzes stderr for a wide range of errors and triggers the correct healing.
     """
+    # Prioritize NumPy 2.0 incompatibility as it's a common, specific failure case.
+    numpy_patterns = [
+        r"A module that was compiled using NumPy 1\.x cannot be run in[\s\S]*?NumPy 2\.0",
+        r"numpy\.dtype size changed, may indicate binary incompatibility"
+    ]
+    for pattern in numpy_patterns:
+        if re.search(pattern, stderr, re.MULTILINE):
+            print(f"\n🔍 NumPy 2.0 compatibility issue detected. Auto-healing with numpy downgrade...")
+            print("   - Downgrading to numpy<2.0 for compatibility")
+            return heal_with_bubble("numpy==1.26.4", Path(cmd_args[0]), cmd_args[1:], config_manager)
+
+    # General patterns for version conflicts, including the AssertionError from your log.
     conflict_patterns = [
         (r"AssertionError: Incorrect ([\w\-]+) version! Expected ([\d\.]+)", 1, 2, "Runtime version assertion"),
         (r"requires ([\w\-]+)==([\d\.]+), but you have", 1, 2, "Import-time dependency conflict"),
         (r"VersionConflict:.*?Requirement\.parse\('([\w\-]+)==([\d\.]+)'\)", 1, 2, "Setuptools VersionConflict")
     ]
-    
-    # Check for version conflicts first
     for regex, pkg_group, ver_group, description in conflict_patterns:
         match = re.search(regex, stderr)
         if match:
@@ -41,32 +50,22 @@ def analyze_runtime_failure_and_heal(stderr: str, cmd_args: list, config_manager
             failed_spec = f"{pkg_name}=={expected_version}"
             print(f"\n🔍 {description} failed. Auto-healing with omnipkg bubbles...")
             print(_("   - Conflict identified for: {}").format(failed_spec))
-            original_script_path = Path(cmd_args[0]).resolve()
-            original_script_args = cmd_args[1:]
-            return heal_with_bubble(failed_spec, original_script_path, original_script_args, config_manager)
-    
-    # Check for missing module errors
+            return heal_with_bubble(failed_spec, Path(cmd_args[0]), cmd_args[1:], config_manager)
+
+    # Fallback to check for completely missing modules.
     missing_module_patterns = [
         (r"ModuleNotFoundError: No module named '([\w\-\.]+)'", 1, "Missing module"),
-        (r"ImportError: No module named ([\w\-\.]+)", 1, "Missing module (ImportError)"),
-        (r"ImportError: cannot import name .+ from '([\w\-\.]+)'", 1, "Missing or incomplete module")
+        (r"ImportError: No module named ([\w\-\.]+)", 1, "Missing module (ImportError)")
     ]
-    
     for regex, pkg_group, description in missing_module_patterns:
         match = re.search(regex, stderr)
         if match:
             module_name = match.group(pkg_group)
-            # Convert module name to likely package name
             pkg_name = convert_module_to_package_name(module_name)
-            print(f"\n🔍 {description} detected. Auto-healing with omnipkg...")
-            print(_("   - Missing module: {}").format(module_name))
-            print(_("   - Attempting to install package: {}").format(pkg_name))
-            original_script_path = Path(cmd_args[0]).resolve()
-            original_script_args = cmd_args[1:]
-            return heal_with_missing_package(pkg_name, original_script_path, original_script_args, config_manager)
-    
+            print(f"\n🔍 {description} detected. Auto-healing by installing missing package...")
+            return heal_with_missing_package(pkg_name, Path(cmd_args[0]), cmd_args[1:], config_manager)
+
     print(_("❌ Script failed with an unhandled runtime error that could not be auto-healed."))
-    print(stderr, file=sys.stderr)
     return 1, None
 
 def convert_module_to_package_name(module_name: str) -> str:
@@ -587,162 +586,81 @@ def heal_with_bubble(required_spec, original_script_path, original_script_args,
     
 def execute_run_command(cmd_args: list, config_manager: ConfigManager):
     """
-    Handles the 'omnipkg run' command by ALWAYS using uv to run the script directly,
-    timing the initial attempt, and catching runtime failures for auto-healing.
+    Handles the 'omnipkg run' command by running the script directly, timing the
+    attempt, and catching both explicit and implicit failures for auto-healing.
     """
     global _initial_run_time_ns
-    
+
     if not cmd_args:
         print(_('❌ Error: No script specified to run.'))
         return 1
 
     print(_(" syncing omnipkg context...")); sync_context_to_runtime(); print(_("✅ Context synchronized."))
-    
+
     python_exe = config_manager.config.get('python_executable', sys.executable)
-    
+
     print(_("🚀 Attempting to run script with uv, forcing use of current environment..."))
     initial_cmd = ['uv', 'run', '--no-project', '--python', python_exe, '--'] + cmd_args
-    
+
     start_time_ns = time.perf_counter_ns()
-    
-    # CRITICAL FIX: Use stderr=subprocess.STDOUT and set bufsize=1 for line buffering
-    # Also add universal_newlines=True for better text handling
+
     process = subprocess.Popen(
-        initial_cmd, 
-        stdout=subprocess.PIPE, 
-        stderr=subprocess.STDOUT,  # Merge stderr into stdout
-        text=True, 
-        encoding='utf-8', 
+        initial_cmd,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.STDOUT,
+        text=True,
+        encoding='utf-8',
         cwd=Path.cwd(),
-        bufsize=1,  # Line buffered
-        universal_newlines=True  # Ensures proper line ending handling
+        bufsize=1,
+        universal_newlines=True
     )
-    
-    # FIXED: Stream output live with immediate flushing
+
     output_lines = []
     try:
-        while True:
-            line = process.stdout.readline()
-            if not line:  # EOF reached
-                break
-            print(line, end='', flush=True)  # CRITICAL: Add flush=True for immediate output
+        # Stream output live for immediate user feedback
+        for line in iter(process.stdout.readline, ''):
+            print(line, end='', flush=True)
             output_lines.append(line)
     except KeyboardInterrupt:
         print("\n🛑 Process interrupted by user")
         process.terminate()
         process.wait()
-        return 130  # Standard exit code for SIGINT
-    
+        return 130
+
     return_code = process.wait()
     end_time_ns = time.perf_counter_ns()
-    
     full_output = "".join(output_lines)
-    
     _initial_run_time_ns = end_time_ns - start_time_ns
 
-    if return_code != 0:
+    # --- Comprehensive Failure Detection ---
+    # Define patterns that indicate a healable error even if the exit code is 0.
+    healable_error_patterns = [
+        r"A module that was compiled using NumPy 1\.x cannot be run in[\s\S]*?NumPy 2\.0",
+        r"numpy\.dtype size changed, may indicate binary incompatibility"
+    ]
+    
+    # Check if a healable error exists in the output, regardless of exit code.
+    has_healable_error = any(re.search(pattern, full_output, re.MULTILINE) for pattern in healable_error_patterns)
+
+    if return_code == 0 and not has_healable_error:
+        print("\n✅ Script executed successfully via uv.")
+        print("⏱️  UV run completed in: {:.3f} ms ({:,} ns)".format(_initial_run_time_ns / 1_000_000, _initial_run_time_ns))
+        return 0
+
+    # If we are here, either the script failed outright or had a detectable issue.
+    if return_code == 0:
+        print("\n🔍 UV succeeded but detected healable errors in output...")
+    else:
         print("⏱️  UV run failed in: {:.3f} ms ({:,} ns)".format(_initial_run_time_ns / 1_000_000, _initial_run_time_ns))
-        
-        # RECURSIVE HEALING: This will now continue until all dependencies are satisfied
-        exit_code, heal_stats = analyze_runtime_failure_and_heal(full_output, cmd_args, config_manager)
-        
-        if heal_stats:
-            _print_performance_comparison(_initial_run_time_ns, heal_stats)
 
-        return exit_code
-
-    print("⏱️  UV run failed in: {:.3f} ms ({:,} ns)".format(_initial_run_time_ns / 1_000_000, _initial_run_time_ns))
-    
+    # Trigger the healing process.
     exit_code, heal_stats = analyze_runtime_failure_and_heal(full_output, cmd_args, config_manager)
-    
+
     if heal_stats:
         _print_performance_comparison(_initial_run_time_ns, heal_stats)
 
     return exit_code
 
-def run_with_healing_wrapper(required_spec, original_script_path, original_script_args, config_manager):
-    """
-    Generates and executes the temporary wrapper script that uses omnipkgLoader.
-    This version captures performance stats and returns them for comparison.
-    """
-    global _initial_run_time_ns
-    
-    # This wrapper script will now print a special JSON line at the very end
-    # containing the performance stats for the parent process to read.
-    wrapper_content = textwrap.dedent(f"""\
-        import sys, os, runpy, json
-        from pathlib import Path
-        sys.path.insert(0, r"{project_root}")
-        from omnipkg.loader import omnipkgLoader
-        from omnipkg.i18n import _
-        
-        lang_from_env = os.environ.get('OMNIPKG_LANG')
-        if lang_from_env: _.set_language(lang_from_env)
-        
-        config = json.loads(r'''{json.dumps(config_manager.config)}''')
-        package_spec = "{required_spec}"
-        
-        loader_instance = None
-        try:
-            print(f"\\n🌀 omnipkg auto-heal: Wrapping script with loader for '{{package_spec}}'...")
-            print('-' * 60)
-            with omnipkgLoader(package_spec, config=config) as loader:
-                loader_instance = loader
-                sys.argv = [{str(original_script_path)!r}] + {original_script_args!r}
-                runpy.run_path({str(original_script_path)!r}, run_name="__main__")
-            print('-' * 60)
-            print(_("✅ Script completed successfully inside omnipkg bubble."))
-        except Exception:
-            import traceback
-            traceback.print_exc()
-            sys.exit(1)
-        finally:
-            if loader_instance:
-                stats = loader_instance.get_performance_stats()
-                if stats:
-                    # Print a machine-readable line for the parent process to capture
-                    print(f"OMNIPKG_STATS_JSON:{json.dumps(stats)}", flush=True)
-    """)
-    
-    temp_script_path = None
-    heal_stats = None
-    try:
-        with tempfile.NamedTemporaryFile(mode='w', suffix='.py', delete=False, encoding='utf-8') as f:
-            f.write(wrapper_content)
-            temp_script_path = f.name
-        
-        heal_command = [config_manager.config.get('python_executable', sys.executable), temp_script_path]
-        
-        print(_("\n🚀 Re-running with omnipkg auto-heal..."))
-        
-        process = subprocess.Popen(heal_command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True, encoding='utf-8')
-        
-        output_lines = []
-        for line in iter(process.stdout.readline, ''):
-            if not line.startswith("OMNIPKG_STATS_JSON:"):
-                print(line, end='') # Print normal output to user in real-time
-            output_lines.append(line)
-        
-        return_code = process.wait()
-
-        # After the process finishes, parse all captured output for the stats line
-        full_output = "".join(output_lines)
-        for line in full_output.splitlines():
-            if line.startswith("OMNIPKG_STATS_JSON:"):
-                try:
-                    stats_json = line.split(":", 1)[1]
-                    heal_stats = json.loads(stats_json)
-                    break
-                except (IndexError, json.JSONDecodeError):
-                    continue
-
-        # --- FIX: Return the captured stats along with the exit code ---
-        return return_code, heal_stats
-
-    finally:
-        if temp_script_path and os.path.exists(temp_script_path):
-            os.unlink(temp_script_path)
-
 def _print_performance_comparison(initial_ns, heal_stats):
     """Prints the final performance summary comparing UV failure time to omnipkg execution time."""
     if not initial_ns or not heal_stats:
@@ -801,4 +719,263 @@ def _print_performance_comparison(initial_ns, heal_stats):
     
     print("="*70)
     print("🌟 Same environment, zero downtime, microsecond swapping!")
-    print("="*70 + "\n")
\ No newline at end of file
+    print("="*70 + "\n")
+    
+def run_with_healing_wrapper(required_spec, original_script_path, original_script_args, config_manager):
+    """
+    Generates and executes the temporary wrapper script. This version creates a
+    robust sys.path in the subprocess, enabling it to find both the omnipkg
+    source and its installed dependencies like 'packaging'.
+    """
+    # PRE-DEBUG: Check what we have available in the current process
+    import site
+    import importlib.util
+    
+    print("\n🔍 PRE-WRAPPER DEBUGGING:")
+    print(f"   Current Python executable: {sys.executable}")
+    print(f"   Current working directory: {os.getcwd()}")
+    print(f"   Project root: {project_root}")
+    
+    # Check if packaging is available in current process
+    try:
+        import packaging
+        print(f"   ✅ packaging found at: {packaging.__file__}")
+    except ImportError as e:
+        print(f"   ❌ packaging not available in current process: {e}")
+    
+    # Get all possible site-packages paths
+    site_packages_paths = []
+    
+    # From config
+    config_site_packages = config_manager.config.get('site_packages_path')
+    if config_site_packages:
+        site_packages_paths.append(config_site_packages)
+        print(f"   Config site-packages: {config_site_packages}")
+    
+    # From site module
+    for path in site.getsitepackages():
+        if path not in site_packages_paths:
+            site_packages_paths.append(path)
+            print(f"   Site getsitepackages: {path}")
+    
+    # From site.USER_SITE
+    if hasattr(site, 'USER_SITE') and site.USER_SITE:
+        if site.USER_SITE not in site_packages_paths:
+            site_packages_paths.append(site.USER_SITE)
+            print(f"   Site USER_SITE: {site.USER_SITE}")
+    
+    # Check current sys.path for site-packages
+    for path in sys.path:
+        if 'site-packages' in path and path not in site_packages_paths:
+            site_packages_paths.append(path)
+            print(f"   Current sys.path site-packages: {path}")
+    
+    # Check each site-packages path for packaging module
+    packaging_locations = []
+    for sp_path in site_packages_paths:
+        if os.path.exists(sp_path):
+            packaging_path = os.path.join(sp_path, 'packaging')
+            packaging_init = os.path.join(sp_path, 'packaging', '__init__.py')
+            if os.path.exists(packaging_path) and os.path.exists(packaging_init):
+                packaging_locations.append(sp_path)
+                print(f"   📦 packaging found in: {sp_path}")
+        else:
+            print(f"   ❌ site-packages path doesn't exist: {sp_path}")
+    
+    if not packaging_locations:
+        print("   ⚠️  WARNING: No packaging module found in any site-packages!")
+    
+    # Use the first site-packages path that has packaging, or fallback to config
+    site_packages_path = packaging_locations[0] if packaging_locations else config_site_packages
+    if not site_packages_path and site_packages_paths:
+        site_packages_path = site_packages_paths[0]
+    
+    print(f"   🎯 Selected site-packages for wrapper: {site_packages_path}")
+
+    # Enhanced wrapper content with comprehensive debugging
+    wrapper_content = textwrap.dedent(f"""\
+        import sys, os, runpy, json
+        from pathlib import Path
+
+        # DEBUGGING: Show initial state
+        print("🔍 WRAPPER SUBPROCESS DEBUGGING:")
+        print(f"   Python executable: {{sys.executable}}")
+        print(f"   Initial sys.path length: {{len(sys.path)}}")
+        print(f"   Working directory: {{os.getcwd()}}")
+        
+        # Show first few sys.path entries
+        for i, path in enumerate(sys.path[:5]):
+            print(f"   sys.path[{{i}}]: {{path}}")
+        if len(sys.path) > 5:
+            print(f"   ... and {{len(sys.path) - 5}} more entries")
+
+        # --- COMPLETE SYS.PATH INJECTION ---
+        project_root_path = r"{project_root}"
+        main_site_packages = r"{site_packages_path}"
+        
+        print(f"\\n   🔧 Adding project root: {{project_root_path}}")
+        if project_root_path not in sys.path:
+            sys.path.insert(0, project_root_path)
+            print(f"      ✅ Added to sys.path[0]")
+        else:
+            print(f"      ⚠️  Already in sys.path")
+
+        print(f"   🔧 Adding site-packages: {{main_site_packages}}")
+        if main_site_packages and main_site_packages not in sys.path:
+            sys.path.insert(1, main_site_packages)
+            print(f"      ✅ Added to sys.path[1]")
+        else:
+            print(f"      ⚠️  Already in sys.path or None")
+        
+        # Add all potential site-packages paths
+        additional_paths = {site_packages_paths!r}
+        print(f"   🔧 Adding {{len(additional_paths)}} additional paths...")
+        for add_path in additional_paths:
+            if add_path and os.path.exists(add_path) and add_path not in sys.path:
+                sys.path.append(add_path)
+                print(f"      ✅ Added: {{add_path}}")
+
+        print(f"\\n   📊 Final sys.path length: {{len(sys.path)}}")
+        
+        # Test critical imports before proceeding
+        print("\\n   🧪 Testing critical imports...")
+        
+        # Test packaging import
+        try:
+            import packaging
+            print(f"      ✅ packaging: {{packaging.__file__}}")
+        except ImportError as e:
+            print(f"      ❌ packaging failed: {{e}}")
+            print("      🔍 Searching for packaging in sys.path...")
+            for i, path in enumerate(sys.path):
+                packaging_path = os.path.join(path, 'packaging')
+                if os.path.exists(packaging_path):
+                    print(f"         Found packaging dir in sys.path[{{i}}]: {{path}}")
+                    init_file = os.path.join(packaging_path, '__init__.py')
+                    print(f"         __init__.py exists: {{os.path.exists(init_file)}}")
+        
+        # Test omnipkg imports
+        try:
+            from omnipkg.loader import omnipkgLoader
+            print(f"      ✅ omnipkgLoader imported")
+        except ImportError as e:
+            print(f"      ❌ omnipkgLoader failed: {{e}}")
+            
+        try:
+            from omnipkg.i18n import _
+            print(f"      ✅ omnipkg.i18n imported")
+        except ImportError as e:
+            print(f"      ❌ omnipkg.i18n failed: {{e}}")
+        # --- END OF PATH INJECTION ---
+
+        # With a correct path, these imports will now succeed.
+        try:
+            from omnipkg.loader import omnipkgLoader
+            from omnipkg.i18n import _
+        except ImportError as e:
+            # This is a fallback error for debugging if the path injection fails.
+            print(f"\\nFATAL: Could not import omnipkg modules after path setup. Error: {{e}}")
+            print(f"\\nDEBUG: Final sys.path ({{len(sys.path)}} entries):")
+            for i, path in enumerate(sys.path):
+                exists = "✅" if os.path.exists(path) else "❌"
+                print(f"   [{{i:2d}}] {{exists}} {{path}}")
+            
+            # Check for omnipkg specifically
+            print(f"\\nDEBUG: Checking for omnipkg module...")
+            for i, path in enumerate(sys.path):
+                omnipkg_path = os.path.join(path, 'omnipkg')
+                if os.path.exists(omnipkg_path):
+                    print(f"   Found omnipkg dir in sys.path[{{i}}]: {{path}}")
+                    loader_file = os.path.join(omnipkg_path, 'loader.py')
+                    print(f"   loader.py exists: {{os.path.exists(loader_file)}}")
+            sys.exit(1)
+
+        lang_from_env = os.environ.get('OMNIPKG_LANG')
+        if lang_from_env: _.set_language(lang_from_env)
+
+        config = json.loads(r'''{json.dumps(config_manager.config)}''')
+        package_spec = "{required_spec}"
+        loader_instance = None
+        try:
+            print(f"\\n🌀 omnipkg auto-heal: Wrapping script with loader for '{{package_spec}}'...")
+            print('-' * 60)
+            with omnipkgLoader(package_spec, config=config) as loader:
+                loader_instance = loader
+                
+                # Debug sys.path after bubble activation
+                print(f"\\n🔍 DEBUG: sys.path after bubble activation ({{len(sys.path)}} entries):")
+                for i, path in enumerate(sys.path[:8]):  # Show first 8 entries
+                    exists = "✅" if os.path.exists(path) else "❌"
+                    print(f"   [{{i}}] {{exists}} {{path}}")
+                if len(sys.path) > 8:
+                    print(f"   ... and {{len(sys.path) - 8}} more entries")
+                
+                # Test critical imports inside bubble
+                print(f"\\n🧪 Testing imports inside bubble...")
+                for module_name in ['packaging', 'filelock', 'toml']:
+                    try:
+                        module = __import__(module_name)
+                        print(f"      ✅ {{module_name}}: {{module.__file__}}")
+                    except ImportError as e:
+                        print(f"      ❌ {{module_name}} failed: {{e}}")
+                        # Try to find it in current sys.path
+                        for i, path in enumerate(sys.path):
+                            if os.path.exists(os.path.join(path, module_name)):
+                                print(f"         Found {{module_name}} in sys.path[{{i}}]: {{path}}")
+                                break
+                            elif os.path.exists(os.path.join(path, f"{{module_name}}.py")):
+                                print(f"         Found {{module_name}}.py in sys.path[{{i}}]: {{path}}")
+                                break
+                
+                print(f"\\n🚀 Running target script...")
+                sys.argv = [{str(original_script_path)!r}] + {original_script_args!r}
+                runpy.run_path({str(original_script_path)!r}, run_name="__main__")
+            print('-' * 60)
+            print(_("✅ Script completed successfully inside omnipkg bubble."))
+        except Exception:
+            import traceback
+            traceback.print_exc()
+            sys.exit(1)
+        finally:
+            if loader_instance:
+                stats = loader_instance.get_performance_stats()
+                if stats:
+                    print(f"OMNIPKG_STATS_JSON:{{json.dumps(stats)}}", flush=True)
+    """)
+
+    temp_script_path = None
+    try:
+        with tempfile.NamedTemporaryFile(mode='w', suffix='.py', delete=False, encoding='utf-8') as f:
+            f.write(wrapper_content)
+            temp_script_path = f.name
+
+        print(f"   💾 Temporary wrapper script: {temp_script_path}")
+
+        heal_command = [config_manager.config.get('python_executable', sys.executable), temp_script_path]
+        print(_("\n🚀 Re-running with omnipkg auto-heal..."))
+
+        process = subprocess.Popen(heal_command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True, encoding='utf-8')
+
+        output_lines = []
+        for line in iter(process.stdout.readline, ''):
+            if not line.startswith("OMNIPKG_STATS_JSON:"):
+                print(line, end='')
+            output_lines.append(line)
+
+        return_code = process.wait()
+        heal_stats = None
+
+        full_output = "".join(output_lines)
+        for line in full_output.splitlines():
+            if line.startswith("OMNIPKG_STATS_JSON:"):
+                try:
+                    stats_json = line.split(":", 1)[1]
+                    heal_stats = json.loads(stats_json)
+                    break
+                except (IndexError, json.JSONDecodeError):
+                    continue
+
+        return return_code, heal_stats
+    finally:
+        if temp_script_path and os.path.exists(temp_script_path):
+            os.unlink(temp_script_path)
\ No newline at end of file
